#!/usr/bin/env python3
# -*- encoding: utf-8 -*-

# Author : Sinseman44 <sinseman44@gmail.com>
# Brief : 
# Pour l'utilisation distante (via SSH), Je ne gère pas l'échange de clé publique entre le client et le serveur SSH
# pour ne pas avoir à me soucier du mot de passe.

#############################################################
# Importation de modules externes                           :

import argparse
import logging as log
import os, re, sys

# for ssh command if distant host
import subprocess
# to write result to xml
from xml.dom.minidom import Document

#############################################################
# Class et Methods                                          :

class App:
    """ Application class """

    def __init__(self, basedir="", parser=None):
        """ Class constructor
        """
        self.parser = parser
        self.basedir = basedir
        self.vers_majeur = '1'
        self.vers_mineur = '0'
        self.xmlfile = 'etat_raid.xml'
        self.command = ['cat', '/proc/mdstat']
        self.pattern_ip = r'^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$'
        self.pattern_port = r'^[0-9]+$'
        self.pattern_line_1 = r'^(md[0-9A-Za-z]+) : (active|inactive) (raid[0-9]) (.*)$'
        self.pattern_line_2 = r'^[ ]{5} ([0-9]+) (.*) \[([0-9]{1,2})/([0-9]{1,2})\] (\[[U|_]+\])$'
        self.doc = Document()
        self.root = self.doc.createElement('root')
        self.doc.appendChild(self.root)
   
    def __write_to_xml(self):
        """ Write XML node to file
        """
        try:
            with open(self.basedir + "/" + self.xmlfile, "w") as file:
                self.doc.writexml(file, indent="    ", newl="\n", encoding="utf-8")
                print("Création du XML (" + self.basedir + "/" + self.xmlfile + ")")
                file.close()
        except (IOError, OSError):
            print("Impossible d'ouvrir le fichier (" + self.basedir + "/" + self.xmlfile + ") en écriture")

    def __create_raid_xml(self, result=[]):
        """ Create XML with mdstat values
        """
        if result == []:
            return

        index = 0
        global_infos = ["cluster_{}_name", "cluster_{}_state", "cluster_{}_type"]
        device_infos = ["cluster_{}_max_devices", "cluster_{}_faulty_devices"]
        md = re.compile(self.pattern_line_1)
        md2 = re.compile(self.pattern_line_2)
        for line in result:
            #print(line.decode('utf-8'))
            # Ligne : => md1 : active raid1 sdb2[1] sda2[0]
            if md.match(line):
                index += 1
                #cluster = self.doc.createElement('cluster_' + str(index))
                #self.root.appendChild(cluster)
                analyse = md.match(line)
                for idx, info in enumerate(global_infos):
                    node = self.doc.createElement(info.format(index))
                    self.root.appendChild(node)
                    text = self.doc.createTextNode(analyse.groups()[idx])
                    node.appendChild(text)
            # Ligne : => 1465151808 blocks level 5, 64k chunk, algorithm 2 [4/3] [UUU_]
            elif md2.match(line):
                analyse = md2.match(line)
                node = self.doc.createElement(device_infos[0].format(index))
                text = self.doc.createTextNode(analyse.groups()[2])
                self.root.appendChild(node)
                node.appendChild(text)
                node = self.doc.createElement(device_infos[1].format(index))
                text = self.doc.createTextNode(str(int(analyse.groups()[2]) - int(analyse.groups()[3])))
                self.root.appendChild(node)
                node.appendChild(text)

        #print(self.doc.toprettyxml(indent="    ", newl="\n", encoding="utf-8").decode('utf-8'))
        self.__write_to_xml()


    def __get_raid_state(self, debug=""):
        """ Retreive mdstat values
        """
        if debug:
            with open(debug, "r") as file:
                result = file.readlines()
                #print(result)
                self.__create_raid_xml(result)
                file.close()
            return

        cmd = subprocess.Popen(self.command, 
                shell=False,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        result = [x.decode('utf-8') for x in cmd.stdout.readlines()]
        #print(result)
        if result == []:
            error = cmd.stderr.readlines()
            print(error)
        else:
            self.__create_raid_xml(result)

    def run(self):
        """ Analyse parser and create xml infos with mdstat values
        """
        debug = ""
        if self.parser.version:
            print("raid_infos.py version : {}.{}".format(self.vers_majeur, self.vers_mineur))
            print("sinseman44@gmail.com")
            return
        if self.parser.debug:
            debug = self.parser.debug[0]
        if self.parser.info:
            self.basedir = self.parser.info[0]
            if self.parser.distant:
                ip = re.compile(self.pattern_ip)
                port = re.compile(self.pattern_port)
                if self.parser.ip and self.parser.port and self.parser.user:
                    if ip.match(self.parser.ip[0]) and port.match(self.parser.port[0]):
                        self.command = ["ssh", 
                                "-p " + self.parser.port[0],
                                self.parser.user[0] + "@" + self.parser.ip[0],
                                self.command[0] + " " + self.command[1]]
                else:
                    print("Les options IP et/ou PORT et/ou USER ne sont pas renseignées")
                    return

            self.__get_raid_state(debug)


#############################################################
# Corps principal du programme                              :

def main():
    curdir = os.path.normpath(os.path.abspath(os.path.curdir))
    parser = argparse.ArgumentParser(description='Etats et Infos sur le Raid Logiciel')
    parser.add_argument('--info',
            action='store',
            nargs=1,
            default=None,
            help='Chemin d\'enregistrement du XML de sortie')
    parser.add_argument('--distant',
            action='store_true',
            help='Hote distant')
    parser.add_argument('--ip',
            action='store',
            nargs=1,
            default=None,
            help='IP de l\'Hote distant')
    parser.add_argument('--port',
            action='store',
            nargs=1,
            default='22',
            help='Port SSH de l\'Hote distant')
    parser.add_argument('--user',
            action='store',
            nargs=1,
            default=None,
            help='Identifiant de l\'utilisateur de l\'Hote distant')
    parser.add_argument('--debug',
            action='store',
            nargs=1,
            default=None,
            help='Ne pas utiliser cette option, juste pour les dev')
    parser.add_argument('--version',
            action='store_true',
            help='Version du script')

    parse_args = parser.parse_args()

    App(basedir=curdir, parser=parse_args).run()

if __name__ == '__main__':
    main()
